/***************************************************************************
Procedure to compute some measures of goodness of fit for discrete choice
model

y=vector of dependent vble. {0,1}. Column Vector.
p=Predicted probability of 1. Column vector.
c=cutt off point. p>c means that a 1 will be predicted.
This is a number between 0 and 1.

If c=0, then c=meanc(y)=frequency of ones in y.
**************************************************************************/

proc clasfdcm(y,p,c);
local a,b,p0,p1,n11,n00,n10,n01,n,mask,fmt,resul,pepito;
/****************** Checking consistency of inputs ***********************/
a=sumc((y.==1) .or (y.==0));
    if a ne rows(y);
    "Error, Bad Imput";
    endif;
if c==0; c=meanc(y); endif;
if c < 0 or c >=1; "Error, Bad Input"; endif;
if sumc(p.<0) .or sumc(p.>1) >0; "Error, Bad Input"; endif;

"";
a=meanc(y);
b=meanc(p);
"Frequency of Ones in sample ";;a;
"Mean predicted probability of Ones ";;b;
"";
p0=p.<c; /* Those obs predicted zero */
p1=p.>=c; /* Those obs predicted one  */
N11=sumc(y.*p1); /* Num obs with 1 that were predicted 1 */
N10=sumc(y.*p0); /* Num obs with 1 that were predicted 0 */
N01=sumc((1-y).*p1); /* Num obs with 0 that were predicted 1 */
N00=sumc((1-y).*p0); /* Num obs with 0 that were predicted 0 */
N=rows(y);
"";
Print "--------------------In %-------------------- ";
Print "Real\Pred       Pred=0      Pred=1       Total";
fmt={ "- *.*lg" 10 10,
      "*.*lf" 12 2,
      "*.*lf" 12 2,
      "*.*lf" 12 2 };
mask={0,1,1,1};
mask=mask';
pepito= { OBSY=0, OBSY=1 };
resul=zeros(2,3);
resul[1,1]=n00/n*100;
resul[1,2]=n01/n*100;
resul[1,3]=(1-a)*100;
resul[2,1]=n10/n*100;
resul[2,2]=n11/n*100;
resul[2,3]=a*100;
call printfm(pepito~resul,mask,fmt);
"";
"Cut-off point ";;c;
"If prob1>";;c;;" then 1 is predicted for that obs";
"Percentage correctly predicted: ";;(resul[1,1]+resul[2,2]);
retp(0);
endp;


