/* **********************************************************************
       This is a program to estimate a simple model where:

-not everyone suffers an illness.
-costs depend on severity,
-costs are not observable for those not going to the doctor

Utility =(Y)-severity^(pow)


*************************************************************************/
New;

library optmum;

#include optmum.ext;
#include hessian.src;
#include presen.prg;
#include hessana.prg;
#include estadi.prg;
#include freqtabl.src;
#include clasfdcm.prg;
#include grad2.g;
#include percenti.prg;
#include delta1.prg;
#include anasimp.prg;
#include anacost.prg;
#include dools.prg;

FN varianza(w)=meanc(w^2)-meanc(w)^2;


/******************USER:  Name of Output Files ***************************/

output file = mop42c.out on;
namecoef="c42c.fmt";
namehess="h42c.fmt";
namevcm="v42c.fmt";
flag=0;peq=0;gra=0;scm=0;flagito=0;flagito1=0;flagito2=0;flagito3=0;
sglob=0;

/******************USER: Loading Vbles **********************************/

#include savememo.prg;         /* This reads the name of the variables */
namevar=setvars("panel");     /* Enter name of dataset                */
dataset="panel";              /* Enter name of dataset                */

/* IF YOU CHANGE DATASET, YOU ALSO MUST GO TO SECTION DESCRIPTIVES OF
DATASET, BEFORE DELETING MISSING VALUES. CHANGE ENTRY IN DSTAT         */

/* IF YOU CHANGE DATASET, YOU NEED ALSO TO DO THE SAME CHANGES IN THE
PART DEDICATED TO OUT OF SAMPLE FORECASTING                            */

open f1= ^dataset for read;
call makevars(readr(f1,rowsf(f1)),0,namevar);
close(f1);


/*****************USER: SCALING ****************************************/

proc estandar(w);
retp((w-meanc(w))./stdc(w));
endp;


Disea=estandar(disea);
Soc=estandar(soc);
Ghea=estandar(ghea);
appt=estandar(appt);
avmd=estandar(avmd);
avsp=estandar(avsp);
educper=estandar(educper);
@Age is not standarized because it changes over time and it has 99999@
Age01=age01/100;
Age02=age02/100;
age03=age03/100;
age04=age04/100;
age05=age05/100;
@Income and costs no because they have structural intrepretation@

/*****************USER: MESSAGE ***************************************/
"";
"WE USE Panel Data";
"We USE THE COST FUNCTION LnC=alpha*ln(s+1), without the copayments";
"Risk Aversion is fixed";
"U(Y)-s^power";
"Alpha is fixed ";

/***********************************************************************
We do not clean missing values, because it should not be any, except those
                                in costs
***********************************************************************/


/************* Restricting Risk Aversion ******************/

"";
tbeta=0.00309923;
"The Value of Risk Aversion is ";;tbeta;
"";

/************* Restricting Alpha ************************/
"";
Alpha=0;
"The Value of Alpha is ";;alpha;
"";

/*************************************************************************/
/*****************Part dedicated to Structural model *********************/
/*************************************************************************/


constant=ones(rows(disea),1);

                   /* Severity variables */

@---------Permanent ones, include constant---------@

inisp = { -0.4784  -0.0400 0.1151  0.0482  -0.0482 -0.1837 };

spnom = { constant   ghea  female  disea  site4 site5 };

@---------Temporary ones, they change over time-------@

inist= { -0.2764 -0.0280 -0.0502  };

parast= { age ac78 ac80 };      @Parameter name@

stnom1 = { age01 ac7801 ac8001 };
stnom2 = { age02 ac7802 ac8001 };
stnom3 = { age03 ac7803 ac8003 };
stnom4 = { age04 ac7804 ac8004 };
stnom5 = { age05 ac7805 ac8005 }; 


                 /* Cost variables */

@---------Permanent ones, include constant-----------@

inicp={ 0.9276   -0.1527 -0.0493 0.2426 -0.3078 0.0272 }; 

cpnom={ constant  ghea      appt site2    site5 disea  };


@--------Temporary ones, they change over time -------@

@IN THIS FILE, THERE IS NOT @

@---------------------Rest of parameters-------------@

inirest={ 0.5407 1.1984 -0.9080 0.2037 0.3859 -0.2293 2.4507 };

restnom={ ae be ce an bn cn pow };



/******** Preparing vectors of parameters*******/

_opparnm=spnom~parast~cpnom~restnom;
_opparnm=_opparnm';

initheta=inisp~inist~inicp~inirest;
initheta=initheta';



clear inisp, inist, inicp, inict, inirest;

"*******************************************************************";
"                 Structural Analysis Follows                       ";
"*******************************************************************";


/********************** Preparing variables ****************************/


n1=sumc(valepi.==1); @Number of people with only 1 valid observation@
n2=sumc(valepi.==2); @Number of people with 2 valid observations@
n3=sumc(valepi.==3); @Number of people with 3 valid observations@
n4=sumc(valepi.==4); @number of people with 4 valid observations@
n5=sumc(valepi.==5); @Number of people with 5 valid observations@

ksp=cols(spnom);
kst=cols(stnom1);
kcp=cols(cpnom);
kct=0;

/***************************SEVERITY VARIABLES**************************/


            @Permanent variables for severity@

sp1=selif(mergevar(spnom'),valepi.==1); @For those that only have 1 valid observation@
sp2=selif(mergevar(spnom'),valepi.==2); @For those that have 2 valid observations@
sp3=selif(mergevar(spnom'),valepi.==3); @for those that have 3 valid observations@
sp4=selif(mergevar(spnom'),valepi.==4); @For those that have 4 valid observations@
sp5=selif(mergevar(spnom'),valepi.==5); @For those that have 5 valid observations @


@Temporary severity variables for the first year@

st1_1=selif(mergevar(stnom1'),valepi.==1); @For those that only have 1 valid observation@
st1_2=selif(mergevar(stnom1'),valepi.==2); @For those that have two valid observations@
st1_3=selif(mergevar(stnom1'),valepi.==3); @For those that have three valid observations@
st1_4=selif(mergevar(stnom1'),valepi.==4); @For those that have four valid observations@
st1_5=selif(mergevar(stnom1'),valepi.==5); @For those that have five valid observations@


@Temporary severity variables for the second year@

st2_2=selif(mergevar(stnom2'),valepi.==2); @For those that have only two valid observations@
st2_3=selif(mergevar(stnom2'),valepi.==3); @For those that have three valid observations@
st2_4=selif(mergevar(stnom2'),valepi.==4); @For those that have four valid observations@
st2_5=selif(mergevar(stnom2'),valepi.==5); @For those that have five valid observations@

@Temporary severity variables for the third year@

st3_3=selif(mergevar(stnom3'),valepi.==3); @For those that have only three valid observations@
st3_4=selif(mergevar(stnom3'),valepi.==4); @For those that have four valid observations@
st3_5=selif(mergevar(stnom3'),valepi.==5); @For those that have five valid observations@

@Temporary severity variables for the fourth year@

st4_4=selif(mergevar(stnom4'),valepi.==4); @For those that have four valid observations@
st4_5=selif(mergevar(stnom4'),valepi.==5); @For those that have five valid observations@


@Temporary severity variables for the fifth year@

st5_5=selif(mergevar(stnom5'),valepi.==5); @For those that have five valid observations@


/**************************COST VARIABLES******************************************/

          @Permanent variables for costs@

cp1=selif(mergevar(cpnom'),valepi.==1); @For those that only have 1 valid observation@
cp2=selif(mergevar(cpnom'),valepi.==2); @For those that have 2 valid observations@
cp3=selif(mergevar(cpnom'),valepi.==3); @for those that have 3 valid observations@
cp4=selif(mergevar(cpnom'),valepi.==4); @For those that have 4 valid observations@
cp5=selif(mergevar(cpnom'),valepi.==5); @For those that have 5 valid observations @


@Temporary cost variables for the first year@
@Temporary cost variables for the second year@
@Temporary cost variables for the third year@
@Temporary cost variables for the fourth year@
@Temporary cost variables for the fifth year@


/*********************************BINARY DEPENDENT VARIABLE*******************************/  

@Binary dependent variables for the first year@

y1_1=selif(depen01,valepi.==1); @For those that only have 1 valid observation@
y1_2=selif(depen01,valepi.==2); @For those that have two valid observations@
y1_3=selif(depen01,valepi.==3); @For those that have three valid observations@
y1_4=selif(depen01,valepi.==4); @For those that have four valid observations@
y1_5=selif(depen01,valepi.==5); @For those that have five valid observations@


@Binary dependent variable for the second year@

y2_2=selif(depen02,valepi.==2); @For those that have only two valid observations@
y2_3=selif(depen02,valepi.==3); @For those that have three valid observations@
y2_4=selif(depen02,valepi.==4); @For those that have four valid observations@
y2_5=selif(depen02,valepi.==5); @For those that have five valid observations@

@Binary dependent variable for the third year@

y3_3=selif(depen03,valepi.==3); @For those that have only three valid observations@
y3_4=selif(depen03,valepi.==4); @For those that have four valid observations@
y3_5=selif(depen03,valepi.==5); @For those that have five valid observations@

@Binary dependent variable for the fourth year@

y4_4=selif(depen04,valepi.==4); @For those that have four valid observations@
y4_5=selif(depen04,valepi.==5); @For those that have five valid observations@


@Binary dependent variable for the fifth year@

y5_5=selif(depen05,valepi.==5); @For those that have five valid observations@


/*************************COST VARIABLES*************************************/

@Cost dependent variables for the first year@

c1_1=selif(cost01,valepi.==1); @For those that onlc have 1 valid observation@
c1_2=selif(cost01,valepi.==2); @For those that have two valid observations@
c1_3=selif(cost01,valepi.==3); @For those that have three valid observations@
c1_4=selif(cost01,valepi.==4); @For those that have four valid observations@
c1_5=selif(cost01,valepi.==5); @For those that have five valid observations@


@Cost dependent variable for the second year@

c2_2=selif(cost02,valepi.==2); @For those that have only two valid observations@
c2_3=selif(cost02,valepi.==3); @For those that have three valid observations@
c2_4=selif(cost02,valepi.==4); @For those that have four valid observations@
c2_5=selif(cost02,valepi.==5); @For those that have five valid observations@

@Cost dependent variable for the third year@

c3_3=selif(cost03,valepi.==3); @For those that have only three valid observations@
c3_4=selif(cost03,valepi.==4); @For those that have four valid observations@
c3_5=selif(cost03,valepi.==5); @For those that have five valid observations@

@Cost dependent variable for the fourth year@

c4_4=selif(cost04,valepi.==4); @For those that have four valid observations@
c4_5=selif(cost04,valepi.==5); @For those that have five valid observations@


@Cost dependent variable for the fifth year@

c5_5=selif(cost05,valepi.==5); @For those that have five valid observations@

/*****************************INCOME VARIABLE**********************************/

@Income variables for the first year@

in1_1=selif(incppc01,valepi.==1); @For those that onlc have 1 valid observation@
in1_2=selif(incppc01,valepi.==2); @For those that have two valid observations@
in1_3=selif(incppc01,valepi.==3); @For those that have three valid observations@
in1_4=selif(incppc01,valepi.==4); @For those that have four valid observations@
in1_5=selif(incppc01,valepi.==5); @For those that have five valid observations@


@Income variable for the second year@

in2_2=selif(incppc02,valepi.==2); @For those that have only two valid observations@
in2_3=selif(incppc02,valepi.==3); @For those that have three valid observations@
in2_4=selif(incppc02,valepi.==4); @For those that have four valid observations@
in2_5=selif(incppc02,valepi.==5); @For those that have five valid observations@

@Income variable for the third year@

in3_3=selif(incppc03,valepi.==3); @For those that have only three valid observations@
in3_4=selif(incppc03,valepi.==4); @For those that have four valid observations@
in3_5=selif(incppc03,valepi.==5); @For those that have five valid observations@

@Income variable for the fourth year@

in4_4=selif(incppc04,valepi.==4); @For those that have four valid observations@
in4_5=selif(incppc04,valepi.==5); @For those that have five valid observations@


@Income variable for the fifth year@

in5_5=selif(incppc05,valepi.==5); @For those that have five valid observations@


/***************************COPAYMENT VARIABLE***********************************/

@Copayment variables for the first year@

k1_1=selif(Copay01,valepi.==1); @For those that onlc have 1 valid observation@
k1_2=selif(Copay01,valepi.==2); @For those that have two valid observations@
k1_3=selif(Copay01,valepi.==3); @For those that have three valid observations@
k1_4=selif(Copay01,valepi.==4); @For those that have four valid observations@
k1_5=selif(Copay01,valepi.==5); @For those that have five valid observations@


@Copayment variable for the second year@

k2_2=selif(Copay02,valepi.==2); @For those that have only two valid observations@
k2_3=selif(Copay02,valepi.==3); @For those that have three valid observations@
k2_4=selif(Copay02,valepi.==4); @For those that have four valid observations@
k2_5=selif(Copay02,valepi.==5); @For those that have five valid observations@

@Copayment variable for the third year@

k3_3=selif(Copay03,valepi.==3); @For those that have only three valid observations@
k3_4=selif(Copay03,valepi.==4); @For those that have four valid observations@
k3_5=selif(Copay03,valepi.==5); @For those that have five valid observations@

@Copayment variable for the fourth year@

k4_4=selif(Copay04,valepi.==4); @For those that have four valid observations@
k4_5=selif(Copay04,valepi.==5); @For those that have five valid observations@


@Copayment variable for the fifth year@

k5_5=selif(Copay05,valepi.==5); @For those that have five valid observations@


/******************  FAKE COPAYMENT VARIABLE***********************************/

PROC pFAKEK(K);
RETP(k.*(k .gt 0.001)+0.6*(k .le 0.001));
ENDP;

@FAKE Copayment variables for the first year@

Fk1_1=pFAKEK(K1_1); @For those that onlc have 1 valid observation@
Fk1_2=pFAKEK(K1_2); @For those that have two valid observations@
Fk1_3=pFAKEK(K1_3); @For those that have three valid observations@
Fk1_4=pFAKEK(K1_4); @For those that have four valid observations@
Fk1_5=pFAKEK(K1_5); @For those that have five valid observations@


@FAKE Copayment variable for the second year@


Fk2_2=pFAKEK(K2_2); @For those that have two valid observations@
Fk2_3=pFAKEK(K2_3); @For those that have three valid observations@
Fk2_4=pFAKEK(K2_4); @For those that have four valid observations@
Fk2_5=pFAKEK(K2_5); @For those that have five valid observations@

@FAKE Copayment variable for the third year@


Fk3_3=pFAKEK(K3_3); @For those that have three valid observations@
Fk3_4=pFAKEK(K3_4); @For those that have four valid observations@
Fk3_5=pFAKEK(K3_5); @For those that have five valid observations@

@FAKE Copayment variable for the fourth year@


Fk4_4=pFAKEK(K4_4); @For those that have four valid observations@
Fk4_5=pFAKEK(K4_5); @For those that have five valid observations@


@FAKE Copayment variable for the fifth year@

Fk5_5=pFAKEK(K5_5); @For those that have five valid observations@


/*********************FAKE COST VARIABLE****************************/

proc pFAKEC(c,y);
local n,itemp,resul;
n=rows(y);
itemp=0;
resul=999*ones(n,1);
do until itemp==n;
    itemp=itemp+1;
        if y[itemp]==1;
        resul[itemp]=c[itemp];
        endif;
endo;
retp(resul);
endp;

@FAKE cost variable for the first year@

FC1_1=pFAKEC(c1_1,y1_1); @For those that onlc have 1 valid observation@
FC1_2=pFAKEC(c1_2,y1_2); @For those that have two observations@
FC1_3=pFAKEC(C1_3,y1_3); @For those that have three observations@
FC1_4=pFAKEC(C1_4,Y1_4); @For those that have four observations@
FC1_5=pFAKEC(C1_5,Y1_5); @For those that have five observations@

@FAKE cost variable for the second year@

FC2_2=pFAKEC(c2_2,y2_2); @For those that have two observations@
FC2_3=pFAKEC(C2_3,y2_3); @For those that have three observations@
FC2_4=pFAKEC(C2_4,Y2_4); @For those that have four observations@
FC2_5=pFAKEC(C2_5,Y2_5); @For those that have five observations@

@FAKE cost variable for the third year@

FC3_3=pFAKEC(C3_3,y3_3); @For those that have three observations@
FC3_4=pFAKEC(C3_4,Y3_4); @For those that have four observations@
FC3_5=pFAKEC(C3_5,Y3_5); @For those that have five observations@

@FAKE cost variable for the fourth year@

FC4_4=pFAKEC(C4_4,Y4_4); @For those that have four observations@
FC4_5=pFAKEC(C4_5,Y4_5); @For those that have five observations@

@FAKE cost variable for the fifth year@

FC5_5=pFAKEC(C5_5,Y5_5); @For those that have five observations@



/****************************************************************************
           Clearing some vbles to leave space free
****************************************************************************/

#include savememo.prg /* This cleans from memory, previous varaibles from the dataset */

clear peq, gra, glo1,glo2,glo3,glo4,glo5,glo6;


/******************* OBTAINNING RANDOM DRAWS ***************/
rndseed 3;
Rn=40;   @Total number of random draws for the normal. It should be "PAR"@      
Ru=40;   @Total number of random draws for the uniform. It should be "PAR"@
"Random draws for Normal and Uniform ";
"Rn ";;rn;
"Ru ";;ru;

@!!!!!!!!THIS IS NEW !!!!!!!!!!!!!!!@

u1_1=unifanti(n1,Ru); @Uniform draws for one with 1 valid observation@

u1_2=unifanti(n2,Ru); @Uniform draws for people with 2 valid observations@
u2_2=unifanti(n2,Ru);

u1_3=unifanti(n3,Ru); @Uniform draws for people with 3 valid observations@
u2_3=unifanti(n3,Ru);
u3_3=unifanti(n3,Ru);

u1_4=unifanti(n4,Ru); @Uniform draws for people with 4 valid observations@
u2_4=unifanti(n4,Ru);
u3_4=unifanti(n4,Ru);
u4_4=unifanti(n4,Ru);

u1_5=unifanti(n5,Ru); @Uniform draws for people with 5 valid observations@
u2_5=unifanti(n5,Ru);
u3_5=unifanti(n5,Ru);
u4_5=unifanti(n5,Ru);
u5_5=unifanti(n5,Ru);


unos1=ones(n1,Ru);
unos2=ones(n2,Ru);
unos3=ones(n3,Ru);
unos4=ones(n4,Ru);
unos5=ones(n5,Ru);

{ ns1,nc1 }=binoanti(n1,Rn); @Normal draws for people with 1 valid observation@
{ ns2,nc2 }=binoanti(n2,Rn); @Normal draws for people with 2 valid observation@
{ ns3,nc3 }=binoanti(n3,Rn); @Normal draws for people with 3 valid observation@
{ ns4,nc4 }=binoanti(n4,Rn); @Normal draws for people with 4 valid observation@
{ ns5,nc5 }=binoanti(n5,Rn); @Normal draws for people with 5 valid observation@



/*************************** OPTIMIZATION *****************************/
_opgtol=1e-6;
_opalgr=2;
__output=1;
print "----------------Initial conditions are: -----------------------";

call presen2(_opparnm,initheta);
add=1;
"Initial value of the loglikelihood ";
loglike(initheta);


_opmiter=28;
{thetaest,nf,g,retcode}=optprt(optmum(&loglike,initheta));


save ^namecoef=thetaest;
"The Resulting Variance Covariance of individual effects is: SigmaS, SigmaC, Rho";
sigma(thetaest[rows(thetaest)-3],thetaest[rows(thetaest)-2],
thetaest[rows(thetaest)-1]);
"";
"The Resulting Variance Covariance of Epsilon is: SigmaS, SigmaC, Rho";
sigma(thetaest[rows(thetaest)-6],thetaest[rows(thetaest)-5],
thetaest[rows(thetaest)-4]);

"";

" The Risk Aversion was fixed at ";;tbeta;
if retcode ne 0;
   "***********************************************************************
   Retcode is different from zero. It is";;retcode;
   
   endif;



add=0;
jac=grad2(&loglike,thetaest,loglike(thetaest),0); /* Jacobian */
add=1;
inf=jac'*jac;  /*Information matrix */
/*
invhes=invpd(hessiano);
*/
invinf=invpd(inf);
/*
vcm1=invhes*inf*invhes;  /*sandwich form */
vcm2=invhes;            /*Using the hessian */
*/
vcm3=invinf;            /* outer product of gradients */
save ^namevcm=vcm3;
/*
std1=sqrt(diag(vcm1));
std2=sqrt(diag(vcm2));
*/
std3=sqrt(diag(vcm3));
"";
/*
presen3(_opparnm,thetaest,std1,std2,std3,thetaest./std1);
*/
"";
"";
"T-Statistics with the Outer product of the gradient";
presen(_opparnm,thetaest,std3,thetaest./std3);
"";
"";
"Using out of product of the gradients...";
"Standard deviation of correlation of unobservables of the random effect (delta method):";
standrho(thetaest[rows(thetaest)-2],thetaest[rows(thetaest)-1],
vcm3[rows(thetaest)-2:rows(thetaest)-1,rows(thetaest)-2:rows(thetaest)-1]);
"";
"Standard deviation of correlation of unobservables of the Epsilons (delta method):";
standrho(thetaest[rows(thetaest)-6],thetaest[rows(thetaest)-5],
vcm3[rows(thetaest)-6:rows(thetaest)-5,rows(thetaest)-6:rows(thetaest)-5]);
"";

/**************************************************************************
                    Procedures to estimate
***************************************************************************/

/******************Log-likelihood Function******************************/ 

proc loglike(theta);
local kreg, betasp, betast, betacp, betact, sigs, sigc, rho, anu, bnu, cnu, pow;
local niter, ns, nc, addnow,lf;

local acu1_1, spbet1, stbet1_1, cpbet1, acu1_1, relxs1_1, fibs1_1, relxc1_1;

local acu1_2, spbet2, stbet1_2, cpbet2, acu1_2, relxs1_2, fibs1_2, relxc1_2;
local  acu2_2, stbet2_2, acu2_2, relxs2_2, fibs2_2, relxc2_2;

local acu1_3, spbet3, stbet1_3, cpbet3, acu1_3, relxs1_3, fibs1_3, relxc1_3;
local  acu2_3, stbet2_3, acu2_3, relxs2_3, fibs2_3, relxc2_3;
local  acu3_3, stbet3_3, acu3_3, relxs3_3, fibs3_3, relxc3_3;

local  acu1_4, spbet4, stbet1_4, cpbet4, acu1_4, relxs1_4, fibs1_4, relxc1_4;
local  acu2_4, stbet2_4, acu2_4, relxs2_4, fibs2_4, relxc2_4;
local  acu3_4, stbet3_4, acu3_4, relxs3_4, fibs3_4, relxc3_4;
local  acu4_4, stbet4_4, acu4_4, relxs4_4, fibs4_4, relxc4_4;

local  acu1_5, spbet5, stbet1_5, cpbet5, acu1_5, relxs1_5, fibs1_5, relxc1_5;
local  acu2_5, stbet2_5,  acu2_5, relxs2_5, fibs2_5, relxc2_5;
local  acu3_5, stbet3_5, acu3_5, relxs3_5, fibs3_5, relxc3_5;
local  acu4_5, stbet4_5, acu4_5, relxs4_5, fibs4_5, relxc4_5;
local  acu5_5, stbet5_5, acu5_5, relxs5_5, fibs5_5, relxc5_5;


/* NEW */
local sbet1_1, cbet1_1, sbet1_2, cbet1_2, sbet2_2, cbet2_2;
local sbet1_3, sbet2_3, sbet3_3, cbet1_3, cbet2_3, cbet3_3;
local sbet1_4, sbet2_4, sbet3_4, sbet4_4, cbet1_4, cbet2_4, cbet3_4, cbet4_4;
local sbet1_5, sbet2_5, sbet3_5, sbet4_5, sbet5_5, cbet1_5, cbet2_5, cbet3_5, cbet4_5, cbet5_5;

local ns1tot,ns2tot,ns3tot,ns4tot,ns5tot;   @NEW@
local nc1tot,nc2tot,nc3tot,nc4tot,nc5tot;


local cont,cont1,cont2,cont3,cont4,cont5;


kreg=ksp+kst+kcp;
betasp=theta[1:ksp];
betast=theta[ksp+1:ksp+kst];
betacp=theta[ksp+kst+1:kreg ];



{sigs,sigc,rho}=sigma(theta[kreg+1],theta[kreg+2],theta[kreg+3]); @Variance covariance parameters of Es,Ec@

anu=theta[kreg+4]; @-----Cholesky parameters of random effects------@
bnu=theta[kreg+5];
cnu=theta[kreg+6];

pow=theta[kreg+7];

@-------------------------------------------------------------
    Compute the likelihood of those with 1 valid episode       
-------------------------------------------------------------@

niter=0;

@Doing regressors for those with one valid observation@

spbet1=sp1*betasp;
stbet1_1=st1_1*betast;
cpbet1=cp1*betacp;


sbet1_1=spbet1+stbet1_1;  @THIS IS NEW@
cbet1_1=cpbet1;

acu1_1=zeros(n1,1);  @This will pick up the contributions for each random effect draw@

ns1tot=anu*ns1;                      @NEW@
nc1tot=bnu*ns1+cnu*nc1;

do until niter==Rn;  @This loop is to compute the contributions for each random effect draw@

    niter=niter+1;

    @Updating random effects draws with cholesky paramenters@
    
    ns=ns1tot[.,niter];               @NEW@
    nc=nc1tot[.,niter];

    relxs1_1=sbet1_1+ns;
    fibs1_1=cdfn(-relxs1_1/sigs);
    relxc1_1=cbet1_1+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs1_1,fibs1_1,relxc1_1,in1_1,k1_1,fk1_1,fc1_1,unos1,u1_1,y1_1);
    
    acu1_1=acu1_1~addnow;

endo;

acu1_1=acu1_1[.,2:cols(acu1_1)]; @Taking the columns of zeros out@    

cont1=meanc(acu1_1'); @Averaging over the random effect draws@
                      @This is the contribution for those with only 1 valid episode@

clear relxs1_1,fibs1_1,relxc1_1,acu1_1,addnow;

@-------------------------------------------------------------
    Compute the likelihood of those with 2 valid episode       
-------------------------------------------------------------@

niter=0;

@Doing regressors for those with two valid observation@

spbet2=sp2*betasp;
stbet1_2=st1_2*betast;
stbet2_2=st2_2*betast;

cpbet2=cp2*betacp;

sbet1_2=spbet2+stbet1_2;
sbet2_2=spbet2+stbet2_2;
cbet1_2=cpbet2;
cbet2_2=cpbet2;

acu1_2=zeros(n2,1);  @This will pick up the contributions for each random effect draw, for the first year@
acu2_2=zeros(n2,1);  @This will pick up the contributions for each random effect draw, for the second year@

ns2tot=anu*ns2;                      @NEW@
nc2tot=bnu*ns2+cnu*nc2;

do until niter==Rn;  @This loop is to compute the contributions for each random effect draw@
                     @And for each period of time=1,2@
    niter=niter+1;

    @Updating random effects draws with cholesky paramenters@
    
    ns=ns2tot[.,niter];
    nc=nc2tot[.,niter];

    @T=1@
    relxs1_2=sbet1_2+ns;
    fibs1_2=cdfn(-relxs1_2/sigs);
    relxc1_2=cbet1_2+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs1_2,fibs1_2,relxc1_2,in1_2,k1_2,fk1_2,fc1_2,unos2,u1_2,y1_2);
    
    acu1_2=acu1_2~addnow;

    @T=2@
    relxs2_2=sbet2_2+ns;
    fibs2_2=cdfn(-relxs2_2/sigs);
    relxc2_2=cbet2_2+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs2_2,fibs2_2,relxc2_2,in2_2,k2_2,fk2_2,fc2_2,unos2,u2_2,y2_2);
    
    acu2_2=acu2_2~addnow;


endo;

acu1_2=acu1_2[.,2:cols(acu1_2)]; @Taking the columns of zeros out@    
acu2_2=acu2_2[.,2:cols(acu2_2)]; @Taking the columns of zeros out@  

cont2=acu1_2.*acu2_2; @Multipliying the contribution of each period of time, given the random effect@
                      

cont2=meanc(cont2');  @Averaging over the random effect draws@
                      @This is the contribution for those with 2 valid episode@


clear relxs1_2,fibs1_2,relxc1_2,acu1_2,addnow;
clear relxs2_2,fibs2_2,relxc2_2,acu2_2;

@-------------------------------------------------------------
    Compute the likelihood of those with 3 valid episode       
-------------------------------------------------------------@

niter=0;

@Doing regressors for those with 3 valid observation@

spbet3=sp3*betasp;
stbet1_3=st1_3*betast;
stbet2_3=st2_3*betast;
stbet3_3=st3_3*betast;

cpbet3=cp3*betacp;

sbet1_3=spbet3+stbet1_3;
sbet2_3=spbet3+stbet2_3;
sbet3_3=spbet3+stbet3_3;

cbet1_3=cpbet3;
cbet2_3=cpbet3;
cbet3_3=cpbet3;

acu1_3=zeros(n3,1);  @This will pick up the contributions for each random effect draw, for the first year@
acu2_3=zeros(n3,1);  @This will pick up the contributions for each random effect draw, for the second year@
acu3_3=zeros(n3,1);  @This will pick up the contributions for each random effect draw, for the third year@

ns3tot=anu*ns3;                      @NEW@
nc3tot=bnu*ns3+cnu*nc3;

do until niter==Rn;  @This loop is to compute the contributions for each random effect draw@
                     @And for each period of time=1,2,3@
    niter=niter+1;

    @Updating random effects draws with cholesky paramenters@
    
    ns=ns3tot[.,niter];               @NEW@
    nc=nc3tot[.,niter];

    @T=1@
    relxs1_3=sbet1_3+ns;
    fibs1_3=cdfn(-relxs1_3/sigs);
    relxc1_3=cbet1_3+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs1_3,fibs1_3,relxc1_3,in1_3,k1_3,fk1_3,fc1_3,unos3,u1_3,y1_3);
    
    acu1_3=acu1_3~addnow;

    @T=2@
    relxs2_3=sbet2_3+ns;
    fibs2_3=cdfn(-relxs2_3/sigs);
    relxc2_3=cbet2_3+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs2_3,fibs2_3,relxc2_3,in2_3,k2_3,fk2_3,fc2_3,unos3,u2_3,y2_3);
    
    acu2_3=acu2_3~addnow;

    @T=3@
    relxs3_3=sbet3_3+ns;
    fibs3_3=cdfn(-relxs3_3/sigs);
    relxc3_3=cbet3_3+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs3_3,fibs3_3,relxc3_3,in3_3,k3_3,fk3_3,fc3_3,unos3,u3_3,y3_3);
    
    acu3_3=acu3_3~addnow;

endo;

acu1_3=acu1_3[.,2:cols(acu1_3)]; @Taking the columns of zeros out@    
acu2_3=acu2_3[.,2:cols(acu2_3)]; @Taking the columns of zeros out@  
acu3_3=acu3_3[.,2:cols(acu3_3)]; @Taking the columns of zeros out@  

cont3=acu1_3.*acu2_3; @Multipliying the contribution of each period of time, given the random effect@
cont3=cont3.*acu3_3;                      

cont3=meanc(cont3');  @Averaging over the random effect draws@
                      @This is the contribution for those with 3 valid episode@


clear relxs1_3,fibs1_3,relxc1_3,acu1_3,addnow;
clear relxs2_3,fibs2_3,relxc2_3,acu2_3;
clear relxs3_3,fibs3_3,relxc3_3,acu3_3;


@-------------------------------------------------------------
    Compute the likelihood of those with 4 valid episode       
-------------------------------------------------------------@

niter=0;

@Doing regressors for those with 4 valid observation@

spbet4=sp4*betasp;
stbet1_4=st1_4*betast;
stbet2_4=st2_4*betast;
stbet3_4=st3_4*betast;
stbet4_4=st4_4*betast;

cpbet4=cp4*betacp;

sbet1_4=spbet4+stbet1_4;
sbet2_4=spbet4+stbet2_4;
sbet3_4=spbet4+stbet3_4;
sbet4_4=spbet4+stbet4_4;

cbet1_4=cpbet4;
cbet2_4=cpbet4;
cbet3_4=cpbet4;
cbet4_4=cpbet4;


acu1_4=zeros(n4,1);  @This will pick up the contributions for each random effect draw, for the first year@
acu2_4=zeros(n4,1);  @This will pick up the contributions for each random effect draw, for the second year@
acu3_4=zeros(n4,1);  @This will pick up the contributions for each random effect draw, for the third year@
acu4_4=zeros(n4,1);  @This will pick up the contributions for each random effect draw, for the third year@

ns4tot=anu*ns4;                      @NEW@
nc4tot=bnu*ns4+cnu*nc4;

do until niter==Rn;  @This loop is to compute the contributions for each random effect draw@
                     @And for each period of time=1,2,3,4@
    niter=niter+1;

    @Updating random effects draws with cholesky paramenters@
    
    ns=ns4tot[.,niter];
    nc=nc4tot[.,niter];

    @T=1@
    relxs1_4=sbet1_4+ns;
    fibs1_4=cdfn(-relxs1_4/sigs);
    relxc1_4=cbet1_4+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs1_4,fibs1_4,relxc1_4,in1_4,k1_4,fk1_4,fc1_4,unos4,u1_4,y1_4);
    
    acu1_4=acu1_4~addnow;

    @T=2@
    relxs2_4=sbet2_4+ns;
    fibs2_4=cdfn(-relxs2_4/sigs);
    relxc2_4=cbet2_4+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs2_4,fibs2_4,relxc2_4,in2_4,k2_4,fk2_4,fc2_4,unos4,u2_4,y2_4);
    
    acu2_4=acu2_4~addnow;

    @T=3@
    relxs3_4=sbet3_4+ns;
    fibs3_4=cdfn(-relxs3_4/sigs);
    relxc3_4=cbet3_4+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs3_4,fibs3_4,relxc3_4,in3_4,k3_4,fk3_4,fc3_4,unos4,u3_4,y3_4);
    
    acu3_4=acu3_4~addnow;

    @T=4@
    relxs4_4=sbet4_4+ns;
    fibs4_4=cdfn(-relxs4_4/sigs);
    relxc4_4=cbet4_4+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs4_4,fibs4_4,relxc4_4,in4_4,k4_4,fk4_4,fc4_4,unos4,u4_4,y4_4);
    
    acu4_4=acu4_4~addnow;

endo;

acu1_4=acu1_4[.,2:cols(acu1_4)]; @Taking the columns of zeros out@    
acu2_4=acu2_4[.,2:cols(acu2_4)]; @Taking the columns of zeros out@  
acu3_4=acu3_4[.,2:cols(acu3_4)]; @Taking the columns of zeros out@  
acu4_4=acu4_4[.,2:cols(acu4_4)]; @Taking the columns of zeros out@

cont4=acu1_4.*acu2_4; @Multipliying the contribution of each period of time, given the random effect@
cont4=cont4.*acu3_4;
cont4=cont4.*acu4_4;                      

cont4=meanc(cont4');  @Averaging over the random effect draws@
                      @This is the contribution for those with 3 valid episode@


clear relxs1_4,fibs1_4,relxc1_4,acu1_4,addnow;
clear relxs2_4,fibs2_4,relxc2_4,acu2_4;
clear relxs3_4,fibs3_4,relxc3_4,acu3_4;
clear relxs4_4,fibs4_4,relxc4_4,acu4_4;

@-------------------------------------------------------------
    Compute the likelihood of those with 5 valid episodes       
-------------------------------------------------------------@

niter=0;

@Doing regressors for those with 5 valid observation@

spbet5=sp5*betasp;
stbet1_5=st1_5*betast;
stbet2_5=st2_5*betast;
stbet3_5=st3_5*betast;
stbet4_5=st4_5*betast;
stbet5_5=st5_5*betast;

cpbet5=cp5*betacp;

sbet1_5=spbet5+stbet1_5;
sbet2_5=spbet5+stbet2_5;
sbet3_5=spbet5+stbet3_5;
sbet4_5=spbet5+stbet4_5;
sbet5_5=spbet5+stbet5_5;

cbet1_5=cpbet5;
cbet2_5=cpbet5;
cbet3_5=cpbet5;
cbet4_5=cpbet5;
cbet5_5=cpbet5;

acu1_5=zeros(n5,1);  @This will pick up the contributions for each random effect draw, for the first year@
acu2_5=zeros(n5,1);  @This will pick up the contributions for each random effect draw, for the second year@
acu3_5=zeros(n5,1);  @This will pick up the contributions for each random effect draw, for the third year@
acu4_5=zeros(n5,1);  @This will pick up the contributions for each random effect draw, for the fourth year@
acu5_5=zeros(n5,1);  @This will pick up the contributions for each random effect draw, for the fifth year@

ns5tot=anu*ns5;                      @NEW@
nc5tot=bnu*ns5+cnu*nc5;

do until niter==Rn;  @This loop is to compute the contributions for each random effect draw@
                     @And for each period of time=1,2,2,3,4,5@
    niter=niter+1;

    @Updating random effects draws with cholesky paramenters@
    
    ns=ns5tot[.,niter];
    nc=nc5tot[.,niter];

    @T=1@
    relxs1_5=sbet1_5+ns;
    fibs1_5=cdfn(-relxs1_5/sigs);
    relxc1_5=cbet1_5+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs1_5,fibs1_5,relxc1_5,in1_5,k1_5,fk1_5,fc1_5,unos5,u1_5,y1_5);
    
    acu1_5=acu1_5~addnow;

    @T=2@
    relxs2_5=sbet2_5+ns;
    fibs2_5=cdfn(-relxs2_5/sigs);
    relxc2_5=cbet2_5+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs2_5,fibs2_5,relxc2_5,in2_5,k2_5,fk2_5,fc2_5,unos5,u2_5,y2_5);
    
    acu2_5=acu2_5~addnow;

    @T=3@
    relxs3_5=sbet3_5+ns;
    fibs3_5=cdfn(-relxs3_5/sigs);
    relxc3_5=cbet3_5+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs3_5,fibs3_5,relxc3_5,in3_5,k3_5,fk3_5,fc3_5,unos5,u3_5,y3_5);
    
    acu3_5=acu3_5~addnow;

    @T=4@
    relxs4_5=sbet4_5+ns;
    fibs4_5=cdfn(-relxs4_5/sigs);
    relxc4_5=cbet4_5+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs4_5,fibs4_5,relxc4_5,in4_5,k4_5,fk4_5,fc4_5,unos5,u4_5,y4_5);
    
    acu4_5=acu4_5~addnow;

    @T=5@
    relxs5_5=sbet5_5+ns;
    fibs5_5=cdfn(-relxs5_5/sigs);
    relxc5_5=cbet5_5+nc; 
    
    addnow=itcont(alpha,sigs,sigc,rho,pow,tbeta,relxs5_5,fibs5_5,relxc5_5,in5_5,k5_5,fk5_5,fc5_5,unos5,u5_5,y5_5);
    
    acu5_5=acu5_5~addnow;


endo;

acu1_5=acu1_5[.,2:cols(acu1_5)]; @Taking the columns of zeros out@    
acu2_5=acu2_5[.,2:cols(acu2_5)]; @Taking the columns of zeros out@  
acu3_5=acu3_5[.,2:cols(acu3_5)]; @Taking the columns of zeros out@  
acu4_5=acu4_5[.,2:cols(acu4_5)]; @Taking the columns of zeros out@
acu5_5=acu5_5[.,2:cols(acu5_5)]; @Taking the columns of zeros out@

cont5=acu1_5.*acu2_5; @Multipliying the contribution of each period of time, given the random effect@
cont5=cont5.*acu3_5;
cont5=cont5.*acu4_5;
cont5=cont5.*acu5_5;                      

cont5=meanc(cont5');  @Averaging over the random effect draws@
                      @This is the contribution for those with 3 valid episode@


clear relxs1_5,fibs1_5,relxc1_5,acu1_5,addnow;
clear relxs2_5,fibs2_5,relxc2_5,acu2_5;
clear relxs3_5,fibs3_5,relxc3_5,acu3_5;
clear relxs4_5,fibs4_5,relxc4_5,acu4_5;
clear relxs5_5,fibs5_5,relxc5_5,acu5_5;


@-------------Computing the likelihood---------------------@


cont=cont1|cont2|cont3|cont4|cont5;

lf=ln(cont);

if add==1;

    lf=sumc(lf);

    else;

    lf=-lf;
    
endif;
           /* Notice if add is not equal 1, the procedure will return the
           vector of individual contributions to the log-likelihood (not
           the minus log likelihood) */

retp(-lf);
endp;


/*************One period-individual contribution to the likelihood******/
          
proc itcont(alpha,sigs,sigc,rho,pow,tbeta,xsbet,fibs,xcbet,inc,k,fakek,fakec,unos,u,y);
local eps,indiec,cmean1,cstan1;
local prbigec,lcnt1,lcnt2,lcnt,indis,obsec,cmean2,cstan2,denobsec,lct,lc,fibs2;

/* Lets obtain the no treatment contribution */

@Now we compute the probability of being ill but not to go because Ec 
is too Big. This should be zero for those with K=0 @

eps=cdfni(fibs.*(unos-u)+u);  @This obtains the truncated Eps from the@
eps=sigs*eps;                              @Uniform draws@
                                           @Notice Eps is NxR @



eps=eps.*(eps .ge -xsbet) + (eps .lt -xsbet).*(-xsbet);


indiec=ecind(inc,fakek,tbeta,pow,xcbet,alpha,xsbet,pow,eps,unos); @Notice using fakek@
                                  @This gives us the indiferent Ec to go to the doctor@



{ cmean1,cstan1 }=condis(rho,sigs,sigc,eps); @These are conditional mean and Stand desv @

PrbigEc=1-cdfn((indiec-cmean1)/cstan1);       @Prob Ec>the indiferent one@



lcnt1=meanc(PrbigEc').*(unos[.,1]-fibs);         @This does the integral@
                                                 @To do the integral we must average over@
                                                 @simulations and then weight for the truncation@

lcnt1=lcnt1.*(k .gt 0.001);                      @This contribution is zero for those with zero copay@                                                 
                                                                                                     


lcnt2=fibs; @Probability of not ill@


lcnt=lcnt1+lcnt2;           @Contribution of no treatment@


/* We Obtain the contribution of Treatment and Cost */

indis=sc(inc,k.*fakec,tbeta,pow); @Notice, using Fake C@
                              @This is indiferent level of severity for observed cost@                            
                              
fibs2=cdfn((indis-xsbet)/sigs);


eps=cdfni(fibs2.*(unos-u)+u);                        @This obtains the truncated Eps from the@
eps=sigs*eps;                                        @Uniform draws. Truncation includes indis@
                                                     @Notice Eps is NxR @
obsec=ecobs(fakec,xcbet,alpha,xsbet,eps,unos);          @This gives us the Ec compatible with observed C and Es@

{ cmean2,cstan2 }=condis(rho,sigs,sigc,eps);         @These are conditional mean and Stand desv @

DenObsEc=pdfn((obsec-cmean2)./cstan2)/cstan2;               @Density of the observed Ec@  

lct=meanc(DenObsEc').*(unos[.,1]-fibs2);                    @This does the integral@
                                                            @To do the integral we must average over@
                                                            @simulations and then weight for the trunctation@
lc=y.*lct+(1-y).*lcnt;


retp(lc);
endp;



/* This computes the Mean and StandarDev of Ec conditional on Es  */
proc(2)= condis(rho,sigs,sigc,eps);
local cmean,cest;
cmean=(rho*sigc/sigs)*eps;
cest=sigc*sqrt(1-rho^2);
retp(cmean,cest);
endp;





/* This computes the deterministic part of the cost function: XcBc+alpha*ln(s+1) */

proc fcostd(xcbet,alpha,xsbet,eps,unos);
local a,s,b;
a=xcbet;
s=(xsbet).*unos+eps;
b=a.*unos+alpha*ln(s+1);
retp(b);
endp;




/* This computes the Ec that corresponds to a given observed cost c given
a shock Es                                                           */

proc ecobs(c,xcbet,alpha,xsbet,eps,unos );
local co,a1,a2;
{ co }=fcostd( xcbet,alpha,xsbet,eps,unos );
a1=ln(c);
a2=a1.*unos-co.*unos;
retp(a2);
endp;



/*  For the model WITH Copayment EFFECTS.
This procedure computes the Ec that leaves the consumer INDIFERENT
whether to go or not to go to the doctor.
OBS=It will give error for those with k=0, since the result is +inf */

proc ecind(inc,k,tbeta,pow,xcbet,alpha,xsbet,pow,eps,unos);
local co,s,a1,a2,a3,a4,verdad,mentira;

co=fcostd(xcbet,alpha,xsbet,eps,unos);

s=xsbet.*unos+eps;

@Notice, if s is too small, a1 might be too negative@ 

a1=ln(s^pow+exp(-tbeta*inc).*unos)./(k*tbeta);

a2=inc./k;

verdad=a2.*unos+a1;
mentira=verdad.*(verdad .ge 0)+zeros(rows(verdad),cols(verdad)).*(verdad .lt 0);

a3=ln(mentira);


a4=a3-co;
retp(a4);
endp;



/***********************************************************************
This gives the indiferent severity.
************************************************************************/

proc sc(inc,kc,tbeta,pow);
local a;
a=-exp(-tbeta*inc)+exp(-tbeta*(inc-kc));
retp(a^(1/pow));
endp;


/************************************************************************
-----------Generate Antithetics Uniform Random Draws---------
************************************************************************/

proc unifanti(row,col);
local u,umu;

@col should be "par"@

u=rndu(row,col/2);
umu=1-u;
u=u~umu;
retp(u);
endp;


/************************************************************************
---Generate Antithetics Bivariate Normal(0,1,rho=0) Random Draws----
************************************************************************/

proc (2)=binoanti(row,col);
local u1,umu1,u2,umu2;

@col should be "par"@
@Row and Col should be of each normal@
u1=rndn(row,col/2);
umu1=-u1;
u1=u1~umu1;

@Row and Col should be of each normal@
u2=rndn(row,col/2);
umu2=-u2;
u2=u2~umu2;

retp(u1,u2);
endp;


proc(3)=sigma(a,b,c);
local varis,varic,stans,stanc,rhosito;
varis=a^2;
varic=b^2+c^2;
stans=sqrt(varis);
stanc=sqrt(varic);
rhosito=(a*b)/(stans*stanc);
retp(stans,stanc,rhosito);
endp;



/* You give the elements of the cov mat, and it gives the matrix such that
matrix*rndn(2,n) give a 2xN matrix such that each row is a normal random
variate and they have the proposed cov mat */

proc(1)=givcho(avar1,avar2,xrho);
local a,b,c,mat;
a=sqrt(avar1);
b=xrho*sqrt(avar2);
c=sqrt(avar2-b^2);
mat=a~zeros(1,1);
mat=mat|(b~c);
retp(mat);
endp;

final:
output off;
end;











